/* 
 * File:   main.c
 * Author: ernestepistola
 *
 * Created on March 24, 2015, 1:51 AM
 */

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "src/Configuration_Bits.h"
#include "src/IO_Pins.h"
#include "src/LCD.h"

#include "Images.h"
#include "Logo.h"

#include <GenericTypeDefs.h>
#include <sys/attribs.h>
#include <stdbool.h>
/*
 * 
 */
volatile unsigned int DelayTicks = 0;
void initTMR3(void);

void delay_ms(unsigned short delay);

void PMP_init(void);
void PMP_WRITE(unsigned char data);

/*Macros for LCD pins*/
/*Chip Select*/
#define LCD_SET_CS() LCD_CS=1
#define LCD_CLR_CS() LCD_CS=0

/*Reset Pin*/
#define LCD_SET_RESET() LCD_RESET=1
#define LCD_CLR_RESET() LCD_RESET=0

/*Register Select*/
#define LCD_SET_RS() LCD_RS=1
#define LCD_CLR_RS() LCD_RS=0

/*Read Strobe*/
#define LCD_SET_RD() LCD_RD=1
#define LCD_CLR_RD() LCD_RD=0

/*Write Strobe*/
#define LCD_SET_WR() LCD_WR=1
#define LCD_CLR_WR() LCD_WR=0

void LCD_RESET_DEVICE(void);
void LCD_WRITE_COMMAND(unsigned char registerToModify);
void LCD_WRITE_DATA(unsigned short dataToWrite);
void LCD_WRITE_REG(unsigned char registerIndex, unsigned short dataToWrite);

void LCD_INIT(void);
void LCD_RESET_RAM_ADDRESS(void);
void LCD_FILLSCREEN(unsigned short color);
void LCD_CLEAR_BUFFER(void);
void LCD_DRAW_SCREEN(void);
void LCD_DMA_INIT(void);
void LCD_PRINT_LOGO(void);

#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 320
#define SCREEN_PIXELS SCREEN_WIDTH*SCREEN_HEIGHT

unsigned short color = 0x00;

int main(void) {

    ANSELA = 0x00; ANSELB = 0x00;
    ANSELC = 0x00; ANSELD = 0x00;
    ANSELE = 0x00; ANSELF = 0x00;
    ANSELG = 0x00;

    TRISA = 0x00; TRISB = 0x00;
    TRISC = 0x00; TRISD = 0x00;
    TRISE = 0x00; TRISF = 0x00;
    TRISG = 0x00;
    
    PB2DIVbits.PBDIV = 0x00;
    
    /*Disable system interrupts*/
    asm volatile("di");
    asm volatile("ehb");
    
    /*Enable multi-vector interrupts and set INTCON to 0x00*/
    INTCON = 0;
    INTCONbits.MVEC = 1;       // Enable Multi-vectored Interrupts   
    LATA = 0x00; LATB = 0x00;
    LATC = 0x00; LATD = 0x00;
    LATE = 0x00; LATF = 0x00;
    LATG = 0x00;
    
    initTMR3();
    /*Enable Interrupts*/
    asm volatile("ei");
    
    PMP_init();
    LCD_INIT();
    
    
    LCD_PRINT_LOGO();
    delay_ms(500);
    
    while(true){
        LCD_RESET_RAM_ADDRESS();
        LCD_FILLSCREEN(color);
        delay_ms(1);
        color+=0xFF;
    }
    
}

void initTMR3(void){
    T3CON = 0;
    T3CONbits.TCKPS = 0b011;//1:8 Prescale Value
    IEC0bits.T3IE = 1;
    IFS0bits.T3IF = 0;
    IPC3bits.T3IP = 7;
    IPC3bits.T3IS = 3;
    PR3           = 5000;//default: 40000
    TMR3          = 0;
    T3CONbits.ON = 1;
}

void __ISR(_TIMER_3_VECTOR, ipl7SRS)_InterruptHandler_TMR3(void)
{
    //LCD_RESET = ~LCD_RESET;
    DelayTicks++;
    IFS0CLR = _IFS0_T3IF_MASK;
}

void delay_ms(unsigned short delay){
    volatile unsigned int currentTime;        // 32 bit value
      currentTime = DelayTicks;
      while(delay > 0) {
          if ( currentTime != DelayTicks ) { // the timer has changed maybe by more than 1 tick but who cares
              currentTime = DelayTicks;
              delay--;
          }
      }
}

void PMP_init(void){
    
    PMCONbits.ON = 0;//reset PMP Module
    PMCONbits.SIDL = 0; //Stop in idle mode bit
    PMCONbits.ADRMUX = 0b00;
    //PMP buffer select
    PMCONbits.PMPTTL = 1;//1 = use TTL buffers; 0 = use Schmitt trigger buffers
    PMCONbits.PTRDEN = 1;//enable read strobe
    PMCONbits.PTWREN = 1;//enable write strobe
    //Chip select function
    PMCONbits.CSF = 0b00;// CS1 and CS2 function as address pins
    PMCONbits.ALP = 1;//address latch polarity (not used for OTM2201A)
    //Chip select polarity, unused
    PMCONbits.CS1P = 0;//active low
    PMCONbits.CS2P = 0;//active low
    //Read and Write strobe polarity
    PMCONbits.WRSP = 0;//active low
    PMCONbits.RDSP = 0;//active low

    #ifdef USE_PMP_IRQ
       //PMP Interrupt request configuration
       PMMODEbits.IRQM = 0b01;//enable PMP interrupts
       /*set PMP interrupt priority (0b111 = 7)*/
       IPC32bits.PMPIP = PMP_PRI;
       /*set PMP interrupt sub priority (0b11 = 1)*/
       IPC32bits.PMPIS = PMP_SUB;
       /*Clear PMP interrupt flag and set enable flag*/
       IFS4bits.PMPIF = 0;//clear interrupt flag
       IEC4bits.PMPIE = 1;//enable PMP interrupt
    #else
      PMMODEbits.IRQM = 0b00;//disable interrupts
    #endif

    //PMP Address increment
    PMMODEbits.INCM = 0b00;
    PMMODEbits.MODE16 = 0;  // 8/16 Mode bit
    PMMODEbits.MODE = 0b10; //configured as master mode 2
    //Data setup wait cycles. range: 0-3
    PMMODEbits.WAITB = 0;//0b00;
    //Read/Write strobe wait cycles.range: 0-15
    PMMODEbits.WAITM = 0;//0b0000;
    //Data Hold after Read/Write. range: 0-3
    PMMODEbits.WAITE = 0;//0b00;
    PMAEN = 0x00;
    PMCONbits.ON = 1;

}

void PMP_WRITE(unsigned char data){
    PMDIN = data;
    while(PMMODEbits.BUSY);

}

void LCD_RESET_DEVICE(){
  LCD_SET_RESET();
  LCD_CLR_RESET();delay_ms(50);
  LCD_SET_RESET();delay_ms(50);
}

void LCD_INIT(){
    
    LCD_RESET_DEVICE();       //toggle reset pin
    LCD_SET_RD();             //not reading
    LCD_CLR_CS();             //select chip

    LCD_WRITE_REG(0x11,0x001A);
    LCD_WRITE_REG(0x12,0x3121);
    LCD_WRITE_REG(0x13,0x006C);
    LCD_WRITE_REG(0x14,0x4249);

    LCD_WRITE_REG(0x10,0x0800);
    delay_ms(10);
    LCD_WRITE_REG(0x11,0x011A);
    delay_ms(10);
    LCD_WRITE_REG(0x11,0x031A);
    delay_ms(10);
    LCD_WRITE_REG(0x11,0x071A);
    delay_ms(10);
    LCD_WRITE_REG(0x11,0x0F1A);
    delay_ms(20);
    LCD_WRITE_REG(0x11,0x0F3A);
    delay_ms(30);

    LCD_WRITE_REG(0x01,0x0128);
    LCD_WRITE_REG(0x02,0x0100);
    LCD_WRITE_REG(0x03,0x1030);
    LCD_WRITE_REG(0x07,0x1012);
    LCD_WRITE_REG(0x08,0x0303);
    LCD_WRITE_REG(0x0B,0x1100);
    LCD_WRITE_REG(0x0C,0x0000);
    LCD_WRITE_REG(0x0F,0x1801);
    LCD_WRITE_REG(0x15,0x0020);
    
    /*
    LCD_WRITE_REG(0x50,0x0101);
    LCD_WRITE_REG(0x51,0x0603);
    LCD_WRITE_REG(0x52,0x0408);
    LCD_WRITE_REG(0x53,0x0000);
    LCD_WRITE_REG(0x54,0x0605);
    LCD_WRITE_REG(0x55,0x0406);
    LCD_WRITE_REG(0x56,0x0303);
    LCD_WRITE_REG(0x57,0x0303);
    LCD_WRITE_REG(0x58,0x0010);
    LCD_WRITE_REG(0x59,0x1000);
    */
    
    LCD_WRITE_REG(0x07,0x0012);
    delay_ms(40);

    LCD_WRITE_REG(0x07,0x0013);/*  GRAM Address Set */
    LCD_WRITE_REG(0x07,0x0017);/*  Display Control  DISPLAY ON */

    LCD_WRITE_REG(0x37, 0); //HorizontalStartAddress
    LCD_WRITE_REG(0x36, 239); //HorizontalEndAddress
    LCD_WRITE_REG(0x39, 0); //VerticalStartAddress
    LCD_WRITE_REG(0x38, 319); //VertocalEndAddress
    
    LCD_WRITE_REG(0x20, 0); //GRAM Address Set
    LCD_WRITE_REG(0x21, 0);
    LCD_WRITE_COMMAND(0x22);//write to memory
    
    delay_ms(10);
}

void LCD_WRITE_COMMAND(unsigned char registerToModify){
    LCD_CLR_RS();                       //RS = 0: Command Mode
    PMP_WRITE(registerToModify);        //Write LSB
}

void LCD_WRITE_DATA(unsigned short dataToWrite){
    LCD_SET_RS();                       //RS = 1: Data Mode
    PMP_WRITE(dataToWrite>>8);          //Write MSB
    PMP_WRITE(dataToWrite);             //Write LSB
}

void LCD_WRITE_REG(unsigned char registerIndex, unsigned short dataToWrite){
    LCD_WRITE_COMMAND(registerIndex);
    LCD_WRITE_DATA(dataToWrite);
}

void LCD_RESET_RAM_ADDRESS(void){
    LCD_WRITE_REG(0x20, 0); //GRAM Address Set
    LCD_WRITE_REG(0x21, 0);
    LCD_WRITE_COMMAND(0x22);//write to memory  
}

void LCD_FILLSCREEN(unsigned short color){
    unsigned int x,y;

    LCD_WRITE_REG(0x20, 0); //GRAM Address Set
    LCD_WRITE_REG(0x21, 0);
    LCD_WRITE_COMMAND(0x22);//write to memory  
    
    for(x = 0; x<SCREEN_WIDTH; x++){
        for(y = 0; y<SCREEN_HEIGHT; y++){
        LCD_RS = 1;
        PMP_WRITE(color>>8);          //Write MSB
        PMP_WRITE(color);             //Write LSB
        }
    }

}

void LCD_CLEAR_BUFFER(){
    unsigned int ii= 0;
    for(ii = 0; ii< SCREEN_PIXELS; ii++){
        //Buffer[ii] = 0x00;
    }
}

void LCD_PRINT_LOGO(){
    unsigned long x = 0;
    unsigned short col;

    img_loop:
    if(x != 0xFFFF){
            col = _Logo.image[x];
            LCD_RS = 1;
            PMP_WRITE(col>>8);          //Write MSB
            PMP_WRITE(col);
            x++;
            goto img_loop;
    }

}